<template>
  <win-md class="stock-edit" :title="`${setting.title}报废`" @close="close" :width="'800px'">
    <el-form class="form" ref="ruleForm" :model="formData" :rules="rules" label-width="120px">
      <div class="main-w">
        <el-row :gutter="24" class="headerHeight">
          <el-col :span="12">
            <el-form-item label="机构" prop="agencyName">
              <el-input :value="formData.agencyName" disabled></el-input>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="报废仓库" prop="warehouseId">
              <el-select v-model="formData.warehouseId" placeholder="请选择" style="width: 100%">
                <el-option v-for="item in warehouses" :key="item.id" :label="item.warehouseName" :value="item.id" />
              </el-select>
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="24" class="headerHeight">
          <el-col :span="12">
            <el-form-item label="报废人" prop="operatorName">
              <el-input :value="formData.operatorName" disabled></el-input>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="报废时间" prop="dealTime">
              <el-date-picker
                v-model="formData.dealTime"
                type="datetime"
                value="yyyy-MM-dd HH:mm:ss"
                value-format="yyyyMMddHHmmss"
                placeholder="请选择日期"
                style="width: 100%"
              >
              </el-date-picker>
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="24" class="headerHeight">
          <el-col :span="24">
            <el-form-item label="报废审批手续" prop="uploadFiles">
              <upload ref="uploadRef" :values="fileList" :settings="uploadSettings" @on-change="uploadChange"></upload>
            </el-form-item>
          </el-col>
        </el-row>
        <div class="goods-card" v-for="(goodsItem, goodsIndex) in formData.scrappedGoodsInfo" :key="goodsIndex">
          <el-row :gutter="24">
            <el-col :span="12">
              <el-form-item
                label="物品分类"
                :prop="`scrappedGoodsInfo[${goodsIndex}].baseCategoryIds`"
                :rules="{
                  required: true,
                  message: '请选择',
                  trigger: 'change',
                }"
              >
                <el-cascader
                  v-model="goodsItem.baseCategoryIds"
                  :options="categoryOptions"
                  :props="{ value: 'id' }"
                  @change="categoryChange($event, goodsIndex)"
                  style="width: 100%"
                ></el-cascader>
              </el-form-item>
            </el-col>
            <el-col :span="12">
              <el-form-item
                label="物品名称"
                :prop="`scrappedGoodsInfo[${goodsIndex}].baseGoodsTemplateId`"
                :rules="rules.baseGoodsTemplateId"
              >
                <el-select
                  v-model="goodsItem.baseGoodsTemplateId"
                  placeholder="请先择物品分类"
                  filterable
                  :disabled="!goodsItem.baseCategoryId"
                  style="width: 100%"
                  @change="goodsTemplateChange($event, goodsIndex)"
                >
                  <el-option
                    v-for="item in goodsItem.goodsOptions"
                    :key="item.id"
                    :label="item.goodsName"
                    :value="item.id"
                  />
                </el-select>
              </el-form-item>
            </el-col>
          </el-row>
          <el-row :gutter="24">
            <el-col :span="12">
              <el-form-item
                label="规格型号"
                :prop="`scrappedGoodsInfo[${goodsIndex}].modelsIds`"
                :rules="{
                  required: true,
                  message: '请选择',
                  trigger: 'change',
                }"
              >
                <el-select
                  v-model="goodsItem.modelsIds"
                  multiple
                  placeholder="请先择物品名称"
                  :disabled="!goodsItem.baseCategoryId"
                  @change="modelChange($event, goodsIndex)"
                  @remove-tag="modelRemoveTag($event, goodsIndex)"
                  style="width: 100%"
                >
                  <el-option
                    v-for="item in goodsItem.modelsOptions"
                    :key="item.id"
                    :label="item.modelName"
                    :value="item.id"
                  />
                </el-select>
              </el-form-item>
            </el-col>
          </el-row>
          <el-table :data="goodsItem.scrappedGoodsList" :stripe="true">
            <el-table-column prop="baseGoodsModelsId" label="型号" align="center">
              <template slot-scope="scope">
                {{ getGoodsModelsName(scope.row.baseGoodsModelsId) }}
              </template>
            </el-table-column>
            <el-table-column prop="total" label="剩余数量" align="center">
              <template slot-scope="scope">
                {{ scope.row.total }}
              </template>
            </el-table-column>
            <el-table-column prop="counts" label="报废数量" align="center">
              <template slot-scope="scope">
                <el-input v-model.number="scope.row.counts"></el-input>
              </template>
            </el-table-column>
            <el-table-column prop="scrappedCode" label="报废原因" align="center">
              <template slot-scope="scope">
                <el-select v-model="scope.row.scrappedCode" placeholder="请选择">
                  <el-option
                    v-for="item in scrapReasonOptions"
                    :key="item.dict_code"
                    :label="item.dict_label"
                    :value="item.dict_code"
                  />
                </el-select>
              </template>
            </el-table-column>
          </el-table>
          <div class="btn-group">
            <el-button
              v-if="formData.scrappedGoodsInfo.length > 1"
              name="移除"
              type="danger"
              plain
              size="mini"
              @click="removeGoods(goodsIndex)"
              >移除</el-button
            >
            <el-button
              v-if="formData.scrappedGoodsInfo.length - 1 == goodsIndex"
              name="新增物品"
              type="primary"
              plain
              size="mini"
              @click="addGoods"
              >新增物品</el-button
            >
          </div>
        </div>
      </div>
    </el-form>
    <div slot="footer" align="center" class="dialog-footer">
      <my-button name="取消" site="form" @click="close" />
      <my-button name="保存" site="form" @click="handleSubmit" />
    </div>
  </win-md>
</template>
<script>
import {scrappedAdd} from '@/api/stock/scrap';
import {goodsModel, goodsTemplate, selectTenantWarehouse, warehouseSelectNumber,} from '@/api/baseSetting/finsystenant';
import {getTree} from '@/api/foudation/classification';
import MyButton from '@/components/myButton/myButton';
import winMd from '@/components/win/win-md';
import upload from '@/components/upload/index';
import {getUploadUrl} from '@/utils/base';
import {mapGetters} from 'vuex';
import {getDicts} from '@/api/system/dict/data';

export default {
  components: { MyButton, winMd, upload },
  props: {
    setting: {
      type: Object,
      default: () => {},
    },
  },
  data() {
    // 自定义校验
    // 物品重复校验
    var checkGoodsTemplateId = (rule, value, callback) => {
      if (!value) {
        return callback(new Error('请选择'));
      }
      let temp = this.formData.scrappedGoodsInfo.filter((v) => v.baseGoodsTemplateId == value);
      if (!temp || temp.length != 1) {
        return callback(new Error('已选过此物品'));
      }
      callback();
    };
    return {
      loading: false,
      fileList: [],
      warehouses: [], // 报废仓库列表
      categoryOptions: [], // 物品分类列表
      scrapReasonOptions: [], // 报废原因列表
      formData: {
        uploadFiles: '',
        warehouseId: '', // 报废仓库id
        dealTime: '', // 报废时间
        scrappedGoodsInfo: [],
      },
      goodsItem: {
        baseCategoryIds: '', // 分类编号数组
        baseCategoryId: '', // 分类编号
        baseGoodsTemplateId: '', // 物品模版编号
        goodsTemplateName: '', // 物品模版名称
        sort: '', // 显示顺序
        goodsOptions: [], // 物品列表select
        modelsOptions: [], //规格型号select
        scrappedGoodsList: [], // 物品名称
        modelsIds: [], //规格型号
      },
      rules: {
        warehouseId: [{ required: true, message: '请选择', trigger: 'change' }],
        dealTime: [{ required: true, message: '请选择', trigger: 'change' }],
        buyType: [{ required: true, message: '请选择', trigger: 'change' }],
        uploadFiles: [{ required: true, message: '请上传', trigger: 'change' }],
        baseCategoryIds: [{ required: true, message: '请选择', trigger: 'change' }],
        baseGoodsTemplateId: [{ validator: checkGoodsTemplateId, trigger: ['blur', 'change'] }],
        modelsIds: [{ required: true, message: '请选择', trigger: 'change' }],
      },

      uploadSettings: {
        title: '上传',
        max: 20, // 最大大小，单位M
        num: 10, // 支持上传图片个数
        tip: '', // 提示 默认：`只能上传${this.defaultSettings.num}个${this.defaultSettings.accept}文件，且不超过${this.defaultSettings.max}kb`
        uploadUrl: getUploadUrl(), // 上传路径
        multiple: true, // 是否支持批量上传
        disabled: false, // 是否禁用
        type: 'picture', // text/picture
      },
    };
  },
  computed: {
    ...mapGetters(['userInfo']),
  },
  created() {
    this.init();
  },
  methods: {
    async init() {
      this.getWarehouseList();
      this.getgoodsTemplate();
      this.getgoodsModel();
      await this.getCategoryTree();
      this.formData.scrappedGoodsInfo.push(JSON.parse(JSON.stringify(this.goodsItem)));
      this.formData.operatorId = this.userInfo.id;
      this.formData.operatorName = this.userInfo.userName;
      this.formData.agencyId = this.userInfo.tenantId;
      this.formData.agencyName = this.userInfo.tenantName;
      this.formData.warehouseType = 0;
      getDicts('SCRAP_REASON').then((res) => {
        this.scrapReasonOptions = res;
      });
    },
    async getCategoryTree() {
      // 获取物品分类列表
      const treeRes = await getTree();
      this.categoryOptions = this.removeEmptyChildren(treeRes);
    },

    // 获取报废仓库列表
    getWarehouseList() {
      selectTenantWarehouse({ agencyId: this.userInfo.tenantId })
        .then((res) => {
          this.warehouses = res;
          if (this.warehouses.length && !this.formData.warehouseId) {
            // 默认选中第一个仓库
            this.formData.warehouseId = this.warehouses[0].id;
          }
        })
        .catch((err) => {
          console.log('err', err);
        });
    },

    removeEmptyChildren(arr) {
      arr.forEach((item) => {
        if (!item.children || !item.children.length) {
          delete item.children;
        } else {
          this.removeEmptyChildren(item.children);
        }
      });
      return arr;
    },

    // 获取物品名称列表
    getgoodsTemplate(id, index) {
      goodsTemplate({ categoryId: id || '' }).then((res) => {
        if (index || index == 0) {
          this.$set(this.formData.scrappedGoodsInfo[index], 'goodsOptions', res);
        } else {
          this.goodsTemplatelAll = res;
        }
      });
    },

    // 规格型号
    getgoodsModel(id, index) {
      goodsModel({ goodsTemplatesId: id || '' }).then((res) => {
        if (index || index == 0) {
          this.$set(this.formData.scrappedGoodsInfo[index], 'modelsOptions', res);
        } else {
          this.goodsModelAll = res;
        }
      });
    },

    // 根据物品名称id获取名字
    getGoodsTemplateName(id) {
      let item = this.goodsTemplatelAll.find((v) => v.id == id);
      if (item) {
        return item.goodsName;
      }
      return;
    },

    // 根据规格型号id获取名字
    getGoodsModelsName(id) {
      let item = this.goodsModelAll.find((v) => v.id == id);
      if (item) {
        return item.modelName;
      }
      return;
    },

    // 物品分类选择
    categoryChange(e, index) {
      this.formData.scrappedGoodsInfo[index].goodsOptions = [];
      this.formData.scrappedGoodsInfo[index].baseGoodsTemplateId = '';
      this.formData.scrappedGoodsInfo[index].goodsTemplateName = '';
      this.formData.scrappedGoodsInfo[index].modelsOptions = [];
      this.formData.scrappedGoodsInfo[index].modelsIds = [];
      this.formData.scrappedGoodsInfo[index].scrappedGoodsList = [];

      this.formData.scrappedGoodsInfo[index].baseCategoryId = e[e.length - 1];
      // 根据选中分类请求物品名称列表
      this.getgoodsTemplate(e[e.length - 1], index);
    },

    // 物品名称列表
    goodsTemplateChange(e, index) {
      this.formData.scrappedGoodsInfo[index].modelsOptions = [];
      this.formData.scrappedGoodsInfo[index].modelsIds = [];
      this.formData.scrappedGoodsInfo[index].scrappedGoodsList = [];

      this.formData.scrappedGoodsInfo[index].goodsTemplateName = this.getGoodsTemplateName(e);
      // 根据选中物品名称id获取规格型号列表
      this.getgoodsModel(e, index);
    },

    // 规格型号选择
    async modelChange(e, index) {
      let arr = [...this.formData.scrappedGoodsInfo[index].scrappedGoodsList];
      let str = JSON.stringify(arr);
      for (let item of e) {
        if (str.indexOf(item) == -1) {
          // 获取库存
          let num = await warehouseSelectNumber({
            warehouseId: this.formData.warehouseId,
            baseGoodsModelsId: item,
            warehouseType: 0,
            states: 1,
            buyType: null,
          });
          arr.push({ baseGoodsModelsId: item, total: num, counts: null, scrappedCode: null });
        }
      }
      this.formData.scrappedGoodsInfo[index].scrappedGoodsList = arr;
    },

    // 规格型号移除
    modelRemoveTag(e, index) {
      let arr = this.formData.scrappedGoodsInfo[index].scrappedGoodsList;
      let delIndex = arr.findIndex((v) => v.baseGoodsModelsId == e);
      this.formData.scrappedGoodsInfo[index].scrappedGoodsList.splice(delIndex, 1);
    },

    // 上传
    uploadChange() {
      let arr = this.$refs.uploadRef.fileList;
      this.formData.uploadFiles = JSON.stringify(arr);
    },

    // 点击新增物品
    addGoods() {
      this.formData.scrappedGoodsInfo.push(JSON.parse(JSON.stringify(this.goodsItem)));
    },

    // 点击移除
    removeGoods(index) {
      this.formData.scrappedGoodsInfo.splice(index, 1);
    },

    // 提交
    handleSubmit() {
      this.$refs['ruleForm'].validate((valid) => {
        if (valid) {
          scrappedAdd(this.formData)
            .then((res) => {
              this.$message.success('保存成功！');
              this.close();
              this.$emit('search');
            })
            .catch((err) => {
              console.log('create err', err);
              this.$message.error('保存失败');
            });
        } else {
          this.$message.error('校验未通过，请检查。');
        }
      });
    },

    close() {
      this.formData = {
        warehouseId: '', // 报废仓库id
        dealTime: '', // 报废时间
        scrappedGoodsInfo: [],
      };
      this.$emit('close');
    },
  },
};
</script>
<style scoped lang="scss">
@import url(../../index.scss);
</style>
